
/* 
 * Title       : rcxrc.cxx
 * Description : RCX remote control for AgendaVR3
 * Author      : KAWAMOTO "nanashino" Yasuhisa
 * Email       : nanashi@yk.rim.or.jp
 * Date        : 04/29/2001
 */

/* 
 * su
 * mipsel-linux-g++ \
 *     -I/usr/mipsel-linux/include/ \
 *     rcxrc.cxx \
 *     -L/usr/mipsel-linux/lib/X11 -lXext -lm -lfltk \
 *     -o rcxrx
 * mipsel-linux-strip rcxrc
 * chmod u+s rcxrc
 */

#include <FL/Fl.H>
#include <FL/Fl_Window.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Repeat_Button.H>
#include <FL/Fl_Box.H>
#include <FL/Fl_Timer.H>
#include <stdio.h>
#include <fcntl.h>

#define BIT_WIDTH (416 - 13)	/* 416us = 1/2400bps */

#define MES_1     0x0001
#define MES_2     0x0002
#define MES_3     0x0004
#define MTR_A_FWD 0x0008
#define MTR_B_FWD 0x0010
#define MTR_C_FWD 0x0020
#define MTR_A_RVS 0x0040
#define MTR_B_RVS 0x0080
#define MTR_C_RVS 0x0100
#define PRG_1     0x0200
#define PRG_2     0x0400
#define PRG_3     0x0800
#define PRG_4     0x1000
#define PRG_5     0x2000
#define PRG_STOP  0x4000
#define BEEP      0x8000
#define REPEAT    (MTR_A_FWD   \
                   | MTR_B_FWD \
                   | MTR_C_FWD \
                   | MTR_A_RVS \
                   | MTR_B_RVS \
                   | MTR_C_RVS)

int lirc[256];
int len;
int fdev;
int m[16] =
{
	MES_1,
	MES_2,
	MES_3,
	PRG_1,
	PRG_2,
	PRG_3,
	PRG_4,
	PRG_5,
	PRG_STOP,
	BEEP,
	MTR_A_FWD,
	MTR_B_FWD,
	MTR_C_FWD,
	MTR_A_RVS,
	MTR_B_RVS,
	MTR_C_RVS,
};
Fl_Button *b[16];
Fl_Timer *timer;

void makecode(unsigned char c)
{
	int w;
	int i;
	int f;
	int p;
	
	p = 1;
	f = 0;
	w = BIT_WIDTH;
	for(i = 0; i < 8; i++)
	{
		if(c & 1)
			p ^= 1;
		if((c & 1) == f)
			w += BIT_WIDTH;
		else
		{
			lirc[len++] = w;
			w = BIT_WIDTH;
			f ^= 1;
		}
		c >>= 1;
	}
	
	if((p & 1) == f)
		w += BIT_WIDTH;
	else
	{
		lirc[len++] = w;
		w = BIT_WIDTH;
		f ^= 1;
	}
	
	if(1 == f)
		w += BIT_WIDTH;
	else
	{
		lirc[len++] = w;
		w = BIT_WIDTH;
		f ^= 1;
	}
	
	lirc[len++] = w;
}

void sendmsg(int m)
{
	unsigned char d, s;
	int res;
	
	len = 0;
	makecode( 0x55);
	makecode( 0xFF);
	makecode( 0x00);
	makecode( 0xD2);
	makecode( 0x2D);
	s = 0xD2;
	d = (m >> 8) & 0xFF;
	makecode( d);
	makecode(~d);
	s += d;
	d = m & 0xFF;
	makecode( d);
	makecode(~d);
	s += d;
	makecode( s);
	makecode(~s);
	
	write(fdev, lirc, (len-1)*sizeof(int));
}

void callback(Fl_Widget *, void *)
{
	int i, msg;
	
	msg = 0;
	for(i = 0; i < 16; i++)
	{
		if(b[i]->value())
			msg |= m[i];
	}
	sendmsg(msg);
	
	if(msg & REPEAT)
		timer->value(0.2);
	else
		timer->suspended(!timer->suspended());
}

int main(int argc, char *argv[])
{
	int i;
	
	fdev = open("/dev/lirc", O_WRONLY);
	Fl_Window *window = new Fl_Window(120, 175);
	
	Fl_Box *box = new Fl_Box(FL_NO_BOX, 10, 0, 100, 20, "RCX remote control");
	box->labelsize(11);
	
	b[ 0] = new Fl_Button(  5, 25, 30, 20, "mes1");
	b[ 1] = new Fl_Button( 45, 25, 30, 20, "mes2");
	b[ 2] = new Fl_Button( 85, 25, 30, 20, "mes3");
	b[ 3] = new Fl_Button(  5,100, 30, 20, "prg1");
	b[ 4] = new Fl_Button( 85,100, 30, 20, "prg2");
	b[ 5] = new Fl_Button(  5,125, 30, 20, "prg3");
	b[ 6] = new Fl_Button( 45,125, 30, 20, "prg4");
	b[ 7] = new Fl_Button( 85,125, 30, 20, "prg5");
	b[ 8] = new Fl_Button(  5,150, 30, 20, "stop");
	b[ 9] = new Fl_Button( 85,150, 30, 20, "beep");
	b[10] = new Fl_Button(  5, 50, 30, 20, "Afwd");
	b[11] = new Fl_Button( 45, 50, 30, 20, "Bfwd");
	b[12] = new Fl_Button( 85, 50, 30, 20, "Cfwd");
	b[13] = new Fl_Button(  5, 75, 30, 20, "Arvs");
	b[14] = new Fl_Button( 45, 75, 30, 20, "Brvs");
	b[15] = new Fl_Button( 85, 75, 30, 20, "Crvs");
	for(i = 0; i < 16; i++)
	{
		b[i]->labelsize(11);
		b[i]->box(FL_ROUND_UP_BOX);
		b[i]->callback(callback, 0);
		b[i]->when(FL_WHEN_CHANGED);
	}
	
	timer = new Fl_Timer(FL_HIDDEN_TIMER, 10, 0, 100, 20, 0);
	timer->callback(callback, 0);
	
	window->end();
	window->show(argc, argv);
	
	Fl::run();
	
	close(fdev);
	return(0);
}
